package pro.litvinovg.libreoffice.metadata.views;

import java.awt.BorderLayout;
import java.awt.EventQueue;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import java.awt.GridBagLayout;
import javax.swing.JCheckBox;
import java.awt.GridBagConstraints;
import javax.swing.JButton;
import java.awt.Insets;
import javax.swing.BoxLayout;
import javax.swing.UIManager;
import java.awt.Toolkit;
import javax.swing.JTabbedPane;
import java.awt.GridLayout;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JMenu;
import javax.swing.JTable;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;

import pro.litvinovg.libreoffice.metadata.CustomDocumentProperty;
import pro.litvinovg.libreoffice.metadata.Document;
import pro.litvinovg.libreoffice.metadata.OutlineElement;

import javax.swing.JScrollPane;
import javax.swing.JPopupMenu;
import java.awt.Component;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Vector;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import javax.swing.JToggleButton;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.JLabel;

public class EditorGUI extends JFrame {
	//Components
	private JTable tableDocProps;
	private JTable tableDocCustomProps;

	
	//Component models
	private DefaultTableModel docPropertiesModel;
	private DefaultTableModel docCustomPropsModel;
	private DefaultTableModel outlineTableModel;

	
	private Map<String, String> docProps;
	private ArrayList<CustomDocumentProperty> docCustomProps;
	private Document document = null;
	private JButton btnNewButton;
	private JMenuItem menuItemPupupDocUserProps;
	private ArrayList<OutlineElement> outline;
	private JTable tableOutline;
	private JPanel panelForOutline;


	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		testGUI();
	}
	
	public static void runGUI(Document doc) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Throwable e) {
			e.printStackTrace();
		}
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					EditorGUI frame = new EditorGUI(doc);
					frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
					frame.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}
	
	private static void testGUI() {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Throwable e) {
			e.printStackTrace();
		}
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					EditorGUI frame = new EditorGUI();
					frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
					frame.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}
	public EditorGUI(Document doc) {
		this.document = doc;
		docProps = document.getDocumentProperties();
		docCustomProps = document.getDocumentCustomProperties();
		outline = document.getOutline();
		initComponents();
		createEvents();
	}
	
	public EditorGUI() {
		outline = new ArrayList<OutlineElement>();
		initComponents();
		createEvents();
	}

	private void createEvents() {
		btnNewButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateDocProps();
				document.setDocumentProperties(docProps);
				updateDocCustomProps();
				document.setCustomDocumentProperties(docCustomProps);
			}
		});
		
		menuItemPupupDocUserProps.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				docCustomPropsModel.addRow(new Object[] {null, null});
			}
		});
	}
	private void updateDocCustomProps() {
		Vector dataVector = docCustomPropsModel.getDataVector();
		Enumeration elements = dataVector.elements();
		docCustomProps.clear();
		while (elements.hasMoreElements()) {
			Vector row = (Vector) elements.nextElement();
			String name = (String) row.get(0);
			String value = (String) row.get(1);
			if (!name.isEmpty() && !value.isEmpty()) {
				docCustomProps.add(new CustomDocumentProperty(name, value, "string"));
			}
		}
	}
	
	private void updateDocProps() {
		Vector dataVector = docPropertiesModel.getDataVector();
		Enumeration elements = dataVector.elements();
		docProps.clear();
		while (elements.hasMoreElements()) {
			Vector row = (Vector) elements.nextElement();
			String name = (String) row.get(0);
			String value = (String) row.get(1);
			docProps.put(name, value);
		}
	}
	private void initComponents() {
		setIconImage(Toolkit.getDefaultToolkit().getImage(EditorGUI.class.getResource("/pro/litvinovg/libreoffice/metadata/resources/webcam.png")));
		setTitle("Редактор метаданных");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setBounds(100, 100, 1064, 733);
		getContentPane().setLayout(null);
		getContentPane().setLayout(new GridLayout(0, 1, 0, 0));
		
		JTabbedPane tabbedPane = new JTabbedPane(JTabbedPane.LEFT, JTabbedPane.SCROLL_TAB_LAYOUT);
		tabbedPane.setBounds(125, 138, 913, 546);
		getContentPane().add(tabbedPane);
		
		JPanel panel = new JPanel();
		tabbedPane.addTab("Документ", null, panel, null);
		for (OutlineElement element : outline) {
			JPanel panelForOutline = new JPanel();
			tabbedPane.addTab(element.getName(), null, panelForOutline, null);
		}
		panelForOutline = new JPanel();
		tabbedPane.addTab("секция", null, panelForOutline, null);
		
		tableOutline = new JTable();
		createOutlineTableModel();
		tableOutline.setModel(outlineTableModel);
		tableOutline.putClientProperty("terminateEditOnFocusLost", true);
		tableOutline.setSurrendersFocusOnKeystroke(true);
		tableOutline.setCellSelectionEnabled(true);
		tableOutline.setFillsViewportHeight(true);
		
		GroupLayout gl_panelForOutline = new GroupLayout(panelForOutline);
		gl_panelForOutline.setHorizontalGroup(
			gl_panelForOutline.createParallelGroup(Alignment.LEADING)
				.addGroup(gl_panelForOutline.createSequentialGroup()
					.addGap(34)
					.addComponent(tableOutline, GroupLayout.DEFAULT_SIZE, 804, Short.MAX_VALUE)
					.addGap(126))
		);
		gl_panelForOutline.setVerticalGroup(
			gl_panelForOutline.createParallelGroup(Alignment.LEADING)
				.addGroup(gl_panelForOutline.createSequentialGroup()
					.addGap(85)
					.addComponent(tableOutline, GroupLayout.DEFAULT_SIZE, 475, Short.MAX_VALUE)
					.addGap(120))
		);
		panelForOutline.setLayout(gl_panelForOutline);
		
		tabbedPane.setEnabledAt(0, true);
		
		
		createDocPropsModel();
		createCustomDocPropsModel();
		
		
		tableDocProps = new JTable();
		tableDocProps.putClientProperty("terminateEditOnFocusLost", true);
		tableDocProps.setSurrendersFocusOnKeystroke(true);
		tableDocProps.setCellSelectionEnabled(true);
		tableDocProps.setFillsViewportHeight(true);
		tableDocProps.setModel(docPropertiesModel);
		
		tableDocCustomProps = new JTable();
		tableDocCustomProps.setColumnSelectionAllowed(true);
		tableDocCustomProps.putClientProperty("terminateEditOnFocusLost", true);

		tableDocCustomProps.setSurrendersFocusOnKeystroke(true);
		tableDocCustomProps.setFillsViewportHeight(true);
		tableDocCustomProps.setCellSelectionEnabled(true);
		tableDocCustomProps.setModel(docCustomPropsModel);
		
		btnNewButton = new JButton("Save");
		
		JLabel labelDocProperties = new JLabel("Основные метаданные");
		
		JLabel labelDocCustomProperties = new JLabel("Дополнительные метаданные");
		
		GroupLayout gl_panel = new GroupLayout(panel);
		gl_panel.setHorizontalGroup(
			gl_panel.createParallelGroup(Alignment.TRAILING)
				.addGroup(Alignment.LEADING, gl_panel.createSequentialGroup()
					.addGap(378)
					.addComponent(labelDocProperties, GroupLayout.DEFAULT_SIZE, 260, Short.MAX_VALUE)
					.addGap(421))
				.addGroup(Alignment.LEADING, gl_panel.createSequentialGroup()
					.addContainerGap()
					.addGroup(gl_panel.createParallelGroup(Alignment.LEADING)
						.addGroup(gl_panel.createSequentialGroup()
							.addComponent(tableDocProps, GroupLayout.DEFAULT_SIZE, 940, Short.MAX_VALUE)
							.addContainerGap())
						.addGroup(Alignment.TRAILING, gl_panel.createSequentialGroup()
							.addComponent(btnNewButton)
							.addGap(294))))
				.addGroup(Alignment.LEADING, gl_panel.createSequentialGroup()
					.addGap(350)
					.addComponent(labelDocCustomProperties, GroupLayout.DEFAULT_SIZE, 233, Short.MAX_VALUE)
					.addGap(381))
				.addGroup(Alignment.LEADING, gl_panel.createSequentialGroup()
					.addContainerGap()
					.addComponent(tableDocCustomProps, GroupLayout.DEFAULT_SIZE, 940, Short.MAX_VALUE)
					.addContainerGap())
		);
		gl_panel.setVerticalGroup(
			gl_panel.createParallelGroup(Alignment.LEADING)
				.addGroup(gl_panel.createSequentialGroup()
					.addGap(53)
					.addComponent(labelDocProperties)
					.addGap(18)
					.addComponent(tableDocProps, GroupLayout.PREFERRED_SIZE, 81, GroupLayout.PREFERRED_SIZE)
					.addPreferredGap(ComponentPlacement.UNRELATED)
					.addComponent(labelDocCustomProperties, GroupLayout.PREFERRED_SIZE, 15, GroupLayout.PREFERRED_SIZE)
					.addGap(18)
					.addComponent(tableDocCustomProps, GroupLayout.PREFERRED_SIZE, 322, GroupLayout.PREFERRED_SIZE)
					.addPreferredGap(ComponentPlacement.RELATED, 90, Short.MAX_VALUE)
					.addComponent(btnNewButton)
					.addGap(31))
		);
		
		JPopupMenu popupMenu = new JPopupMenu();
		addPopup(tableDocCustomProps, popupMenu);
		
		menuItemPupupDocUserProps = new JMenuItem("Добавить строку");
		
		popupMenu.add(menuItemPupupDocUserProps);
		panel.setLayout(gl_panel);
		
	
		JMenuBar menuBar = new JMenuBar();
		setJMenuBar(menuBar);
		
		JMenu mnFile = new JMenu("Файл");
		menuBar.add(mnFile);
	}

	private void createDocPropsModel() {
		Object[][] tableArray = new Object[][] {
			{null, null},
			{null, null},
			{null, null},
			{null, null},
			{null, null},
		};
		if (docProps != null) {
			Object[] arrayOfEntries = docProps.entrySet().toArray();
			tableArray = new Object[docProps.size()][2];
			for (int i = 0; i < tableArray.length; i++) {
				Entry<String, String> entry = (Entry<String, String>) arrayOfEntries[i];
				tableArray[i][0] = entry.getKey();
				tableArray[i][1] = entry.getValue();
			}
		}
		docPropertiesModel = new DefaultTableModel(
				tableArray,
				new String[] {
					"Metadata name", "Metadata value"
				});
	}
	
	private void createOutlineTableModel() {
		outlineTableModel = new DefaultTableModel(
				new Object[][] {
					{null, null},
				},
				new String[] {
					"New column", "New column"
				}
			);
	}
	private void createCustomDocPropsModel() {
		Object[][] tableArray = new Object[][] {
			{null, null},
			{null, null},
			{null, null},
			{null, null},
		};
		if (docCustomProps != null) {
			tableArray = new Object[docCustomProps.size()][2];
			for (int i = 0; i < tableArray.length; i++) {
				CustomDocumentProperty curProp = docCustomProps.get(i);
					tableArray[i][0] = curProp.getName();
					tableArray[i][1] = curProp.getValue();	
			}
		}
		docCustomPropsModel = new DefaultTableModel(
				tableArray,
				new String[] {
					"Metadata name", "Metadata value"
				});
	}
	
	
	private static void addPopup(Component component, final JPopupMenu popup) {
		component.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				if (e.isPopupTrigger()) {
					showMenu(e);
				}
			}
			public void mouseReleased(MouseEvent e) {
				if (e.isPopupTrigger()) {
					showMenu(e);
				}
			}
			private void showMenu(MouseEvent e) {
				popup.show(e.getComponent(), e.getX(), e.getY());
			}
		});
	}
}
